<?php

declare(strict_types=1);

namespace Dedi\SyliusMarketingPlugin\Factory\Events;

use Dedi\SyliusMarketingPlugin\ControllerEvent\SelectItemEvent;
use Dedi\SyliusMarketingPlugin\Events\ItemsContainerInterface;
use Dedi\SyliusMarketingPlugin\Events\Search;
use Dedi\SyliusMarketingPlugin\Events\SelectItem;
use Dedi\SyliusMarketingPlugin\Events\ViewItem;
use Dedi\SyliusMarketingPlugin\Events\ViewItemList;
use Dedi\SyliusMarketingPlugin\Factory\ItemFactory;
use Dedi\SyliusMarketingPlugin\Provider\ItemListProviderInterface;
use Sylius\Component\Core\Model\ProductInterface;
use Sylius\Component\Product\Repository\ProductRepositoryInterface;

class NavigationEventFactory
{
    private ItemFactory $itemFactory;
    private ProductRepositoryInterface $productRepository;
    private ItemListProviderInterface $itemListProvider;

    public function __construct(
        ItemFactory $itemFactory,
        ProductRepositoryInterface $productRepository,
        ItemListProviderInterface $itemListProvider
    ) {
        $this->itemFactory = $itemFactory;
        $this->productRepository = $productRepository;
        $this->itemListProvider = $itemListProvider;
    }
    public function search(string $searchTerm): Search
    {
        return new Search($searchTerm);
    }

    public function viewItemList(array $products = []): ViewItemList
    {
        $viewItemList = (new ViewItemList())
            ->setItemListId($this->itemListProvider->getItemListId())
            ->setItemListName($this->itemListProvider->getItemListName())
        ;
        $this->addItems($viewItemList, $products);

        return $viewItemList;
    }

    public function viewItem(ProductInterface $product): ViewItem
    {
        $viewItem = new ViewItem();
        $viewItem->addItem($this->itemFactory->fromProduct($product));

        return $viewItem;
    }

    public function selectItem(SelectItemEvent $selectItemEvent): SelectItem
    {
        $selectItem = new SelectItem();

        $product = $this->productRepository->find($selectItemEvent->productId);

        if (!$product instanceof ProductInterface) {
            return $selectItem;
        }

        $selectItem->addItem($this->itemFactory->fromProduct($product, $selectItemEvent->productPosition));

        return $selectItem;
    }

    private function addItems(ItemsContainerInterface $event, array $products): void
    {
        foreach ($products as $product) {
            $event->addItem(
                $this->itemFactory->fromProduct($product),
            );
        }
    }
}
