<?php

declare(strict_types=1);

namespace Dedi\SyliusMarketingPlugin\Provider;

use Sylius\Component\Locale\Context\LocaleContextInterface;
use Sylius\Component\Taxonomy\Model\TaxonInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;

class ItemListProvider implements ItemListProviderInterface
{
    private const HOMEPAGE_ITEM_LIST_NAME = 'Homepage';

    private TaxonProviderInterface $taxonProvider;
    private LocaleContextInterface $localeContext;
    private RequestStack $requestStack;
    private UrlGeneratorInterface $router;

    public function __construct(
        TaxonProviderInterface $taxonProvider,
        LocaleContextInterface $localeContext,
        RequestStack $requestStack,
        UrlGeneratorInterface $router
    ) {
        $this->taxonProvider = $taxonProvider;
        $this->localeContext = $localeContext;
        $this->requestStack = $requestStack;
        $this->router = $router;
    }

    public function getItemListId(): ?string
    {
        $taxon = $this->taxonProvider->getListTaxon();

        if ($taxon instanceof TaxonInterface) {
            return (string) $taxon->getId();
        }

        return null;
    }

    public function getItemListName(): ?string
    {
        $taxon = $this->taxonProvider->getListTaxon();

        $itemListName = null;
        if ($this->isHomepageReferer()) {
            $itemListName = self::HOMEPAGE_ITEM_LIST_NAME;
        } elseif ($taxon instanceof TaxonInterface) {
            $itemListName = $taxon->getName();
        }

        return $itemListName;
    }

    private function isHomepageReferer(): bool
    {
        $request = $this->requestStack->getCurrentRequest();

        if (!$request instanceof Request) {
            return false;
        }

        $referer = $request->headers->get('referer');

        if (null === $referer) {
            return false;
        }

        return $referer === $this->router->generate(
            'sylius_shop_homepage',
            ['_locale' => $this->localeContext->getLocaleCode()],
            UrlGeneratorInterface::ABSOLUTE_URL,
        );
    }
}
